@@ -12,7 +12,7 @@ use Sys::Syslog qw(:macros);
 use LISM::Storage;
 use Data::Dumper;
 
-our $VERSION = '2.1.1';
+our $VERSION = '2.1.2';
 
 our $lism_master = 'lism_master';
 our $syncrdn = 'cn=sync';
@@ -21,6 +21,7 @@ our $cluster_syncrdn = 'cn=cluster-sync';
 our $syncInfoEntry = "objectClass: dSA\n";
 our $syncInfoAttr = "description";
 our $nosyncAttr = "seeAlso";
+our $syncDataAttr = "ou";
 our $globalLock = 'global.lock';
 our $syncFailLog = 'syncfail';
 our $sizeLimit = 1000000;
@@ -282,7 +283,7 @@ sub search
     if (defined($conf->{sync})) {
         if ($base =~ /^($syncrdn|$master_syncrdn|$cluster_syncrdn),$self->{_config}->{basedn}$/i) {
             # get information of synchronization
-            return $self->_getSyncInfo($base, $scope, $attrOnly, @attrs);
+            return $self->_getSyncInfo($base, $scope, $filterStr, $attrOnly, @attrs);
         }
     }
 
@@ -925,16 +926,21 @@ sub _doHandler
     my $self = shift;
     my ($type, $func, $dname, @args) = @_;
     my $method = $type.'_'.$func;
+    my $rc = LDAP_SUCCESS;
 
     if (!defined($self->{_handler}{$dname})) {
         return 0;
     }
 
     foreach my $hname (keys %{$self->{_handler}{$dname}}) {
-        return $self->{_handler}{$dname}{$hname}->$method(@args);
+        $rc = $self->{_handler}{$dname}{$hname}->$method(@args);
+        if ($rc) {
+            $rc = LDAP_OTHER;
+            last;
+        }
     }
 
-    return 0;
+    return $rc;
 }
 
 sub _checkSync
@@ -975,10 +981,12 @@ sub _doSync
     }
 
     # update the master storage
-    $rc = $self->_doUpdate($func, undef, 0, $dn, @info);
-    if ($rc) {
-        $self->log(level => 'error', message => "Updating master entry($dn) failed: error code($rc)");
-        return($rc, @updated);
+    if ($func ne 'delete') {
+        $rc = $self->_doUpdate($func, undef, 0, $dn, @info);
+        if ($rc) {
+            $self->log(level => 'error', message => "Updating master entry($dn) failed: error code($rc)");
+            return($rc, @updated);
+        }
     }
 
     foreach my $dname (keys %{$sync->{data}}) {
@@ -1138,6 +1146,15 @@ sub _doSync
         }
     }
 
+    # Delete master entry last for ldap rewrite map
+    if ($func eq 'delete') {
+        $rc = $self->_doUpdate($func, undef, 0, $dn, @info);
+        if ($rc) {
+            $self->log(level => 'error', message => "Updating master entry($dn) failed: error code($rc)");
+            return($rc, @updated);
+        }
+    }
+
     if ($sync->{transaction}[0] !~ /^on$/i) {
         $rc = LDAP_SUCCESS;
     }
@@ -1168,12 +1185,14 @@ sub _updateRollback
 sub _getSyncInfo
 {
     my $self = shift;
-    my ($base, $scope, $attrOnly, @attrs) = @_;
+    my ($base, $scope, $filterStr, $attrOnly, @attrs) = @_;
     my $conf = $self->{_lism};
     my $sync = $conf->{sync}[0];
     my $master = $self->{data}{$lism_master};
     my $present_list;
+    my @check_data = ();
     my $syncStatus = '';
+    my %nosync_data;
     my %nosync_entries;
     my %deletedn;
 
@@ -1182,6 +1201,18 @@ sub _getSyncInfo
         return (0, ());
     }
 
+    # get checked data
+    my (@check_dnames) = ($filterStr =~ /\($syncDataAttr=(.*)\)/gi);
+    if (@check_dnames) {
+        foreach my $dname (keys %{$sync->{data}}) {
+            if (grep(/$dname/i, @check_dnames)) {
+                push(@check_data, $dname);
+            }
+        }
+    } else {
+        @check_data = keys %{$sync->{data}};
+    }
+
     my $lock = $self->_lock($self->{_config}->{lockdir}.'/'.$globalLock, 2);
     if (!defined($lock)) {
         $self->log(level => 'crit', message => "Can't get global lock");
@@ -1196,7 +1227,7 @@ sub _getSyncInfo
     $present_list = $self->_getPresentList();
     if ($base !~ /^$cluster_syncrdn/) {
         # check master data
-        foreach my $dname (keys %{$sync->{data}}) {
+        foreach my $dname (@check_data) {
             my $data = $self->{data}{$dname};
             my $sdata = $sync->{data}{$dname};
 
@@ -1262,6 +1293,7 @@ sub _getSyncInfo
                     if (!defined($present_list->{$sbase}{$key})) {
                         # data storage's entry doesn't exist in master storage
                         if ($ops{add}) {
+                            $nosync_data{$dname} = 1;
                             $nosync_entries{lc($dn)} = 1;
                         }
                     } else {
@@ -1293,6 +1325,7 @@ sub _getSyncInfo
                             }
 
                             if ($dvals !~ /^$pvals$/i && $ops{modify}) {
+                                $nosync_data{$dname} = 1;
                                 $nosync_entries{lc($dn)} = 1;
                                 last;
                             }
@@ -1318,7 +1351,7 @@ sub _getSyncInfo
 
     if ($base !~ /^$master_syncrdn/) {
         # check sync data
-        foreach my $dname (keys %{$sync->{data}}) {
+        foreach my $dname (@check_data) {
             my $data = $self->{data}{$dname};
             my $sdata = $sync->{data}{$dname};
 
@@ -1371,6 +1404,7 @@ sub _getSyncInfo
                     if (!defined($present_list->{$sbase}{$key})) {
                         # data storage's entry doesn't exist in master storage
                         if ($ops{delete}) {
+                            $nosync_data{$dname} = 1;
                             $nosync_entries{lc($dn)} = 1;
                         }
                     } elsif ($present_list->{$sbase}{$key} && !defined($syncfilter) ||
@@ -1407,6 +1441,7 @@ sub _getSyncInfo
                             }
 
                             if ($dvals !~ /^$pvals$/i && $ops{modify}) {
+                                $nosync_data{$dname} = 1;
                                 $nosync_entries{lc($dn)} = 1;
                                 last;
                             }
@@ -1424,6 +1459,7 @@ sub _getSyncInfo
                         if (!defined($present_list->{$sbase}{$key}->{sync_present}{$dname}) &&
                             (!defined($syncfilter) ||
                                 LISM::Storage->parseFilter($syncfilter, $present_list->{$sbase}{$key}->{entryStr}))) {
+                            $nosync_data{$dname} = 1;
                             $nosync_entries{lc("$key,$dbase")} = 1;
                         }
                     }
@@ -1438,6 +1474,9 @@ sub _getSyncInfo
         $syncentry = $syncentry."$syncInfoAttr: sync\n";
     } else {
         $syncentry = $syncentry."$syncInfoAttr: nosync\n";
+        foreach my $dname (keys %nosync_data) {
+            $syncentry = $syncentry."$syncDataAttr: $dname\n";
+        }
         foreach my $dn (keys %nosync_entries) {
             $syncentry = $syncentry."$nosyncAttr: $dn\n";
         }
@@ -1454,12 +1493,25 @@ sub _setSyncInfo
     my $sync = $conf->{sync}[0];
     my $master = $self->{data}{$lism_master};
     my $present_list;
+    my @sync_data = ();
     my %deletedn;
     my $rc = LDAP_SUCCESS;
 
     my $modinfo = join(',', @list);
-    if (!($modinfo =~ /REPLACE,$syncInfoAttr,sync/i)) {
-        return LDAP_UNWILLING_TO_PERFORM;
+
+    # get synchronized data
+    my ($sync_dnames) = ($modinfo =~ /DELETE,$syncDataAttr,(.*),?(ADD|DELETE|REPLACE|)/i);
+    if ($sync_dnames) {
+        foreach my $dname (keys %{$sync->{data}}) {
+            if (",$sync_dnames," =~ /$dname,/i) {
+                push(@sync_data, $dname);
+            }
+        }
+    } else {
+        if ($modinfo !~ /REPLACE,$syncInfoAttr,sync/i) {
+            return LDAP_UNWILLING_TO_PERFORM;
+        }
+        @sync_data = keys %{$sync->{data}};
     }
 
     my $lock = $self->_lock($self->{_config}->{lockdir}.'/'.$globalLock, 2);
@@ -1472,7 +1524,7 @@ sub _setSyncInfo
         # get present entry list
         $present_list = $self->_getPresentList();
 
-        foreach my $dname (keys %{$sync->{data}}) {
+        foreach my $dname (@sync_data) {
             my $data = $self->{data}{$dname};
             my $sdata = $sync->{data}{$dname};
 
@@ -1584,6 +1636,7 @@ sub _setSyncInfo
                         # modify entry which isn't equal to data storage
                         my @modlist;
                         my $pocs;
+                        my $entryStr = $present_list->{$sbase}{$key}->{entryStr};
 
                         if (!defined($sobject->{masterattrs})) {
                             @sync_attrs = $self->_unique(($present_list->{$sbase}{$key}->{entryStr} =~ /^([^:]+):/gmi), ($entries[$i] =~ /\n([^:]+):/gi));
@@ -1640,6 +1693,9 @@ sub _setSyncInfo
                                 } else {
                                     push(@modlist, ('DELETE', $attr));
                                 }
+
+                                $entryStr =~ s/$attr: .*\n//gi;
+                                $entryStr = "$entryStr$dvals\n";
                             }
 		        }
 
@@ -1647,6 +1703,7 @@ sub _setSyncInfo
                             if ($ops{modify}) {
                                 $dn = encode('utf8', $dn);
                                 $rc = $self->_doUpdate('modify', $dname, 1, $dn, @modlist);
+                                $present_list->{$sbase}{$key}->{entryStr} = $entryStr;
                             }
                         }
                     }
@@ -1685,7 +1742,7 @@ sub _setSyncInfo
         undef $present_list;
         $present_list = $self->_getPresentList();
 
-        foreach my $dname (keys %{$sync->{data}}) {
+        foreach my $dname (@sync_data) {
             my $data = $self->{data}{$dname};
             my $sdata = $sync->{data}{$dname};
 
@@ -2089,7 +2146,7 @@ sub _writeSyncFail
     }
 
     flock($fd, 2);
-    $ldif = "# ".strftime("%Y%m%d%H%M%S", localtime(time))."\n$dn\nchangetype: $func\n";
+    $ldif = "# ".strftime("%Y%m%d%H%M%S", localtime(time))."\ndn: $dn\nchangetype: $func\n";
 
     if ($func eq 'modify') {
         while (@info > 0) {
