@@ -299,6 +299,16 @@ sub _checkConfig
     return 0;
 }
 
+sub _unique
+{
+    my $self = shift;
+    my @array = @_;
+
+    my %hash = map {$_ => 1} @array;
+
+    return keys %hash;
+}
+
 =head1 SEE ALSO
 
 L<LISM>
