@@ -35,6 +35,10 @@ sub pre_bind
 
     foreach my $rule (@{$conf->{rewrite}}) {
         if ($rule->{context} eq 'request') {
+            if (defined($rule->{dn}) && ${$binddnp} !~ /$rule->{dn}/i) {
+                next;
+            }
+
             my $substitution = $rule->{substitution};
             $substitution =~ s/%0/${$binddnp}/;
 
@@ -64,6 +68,10 @@ sub pre_compare
 
     foreach my $rule (@{$conf->{rewrite}}) {
         if ($rule->{context} eq 'request') {
+            if (defined($rule->{dn}) && ${$dnp} !~ /$rule->{dn}/i) {
+                next;
+            }
+
             my %rwcache;
             my $substitution = $rule->{substitution};
             $substitution =~ s/%0/${$dnp}\n${$avaStrp}/;
@@ -146,6 +154,9 @@ sub post_search
 
                 my (@line) = split(/\n/, ${$entriesp}[$i]);
                 my ($dn) = ($line[0] =~ /^dn: (.*)$/);
+                if (defined($rule->{dn}) && $dn !~ /$rule->{dn}/i) {
+                    next;
+                }
 
                 $line[0] = $self->_rewriteParse($rule->{match}, $substitution, $line[0], \%rwcache);
                 if (!$line[0]) {
@@ -190,6 +201,10 @@ sub pre_modify
 
     foreach my $rule (@{$conf->{rewrite}}) {
         if ($rule->{context} eq 'request') {
+            if (defined($rule->{dn}) && ${$dnp} !~ /$rule->{dn}/i) {
+                next;
+            }
+
             my %rwcache;
             my $substitution = $rule->{substitution};
             my $modlist = "${$dnp}\n".join("\n", @{$listp});
@@ -206,7 +221,7 @@ sub pre_modify
             while (@{$listp} > 0) {
                 my $action = shift @{$listp};
                 my $attr = shift @{$listp};
-                my @replaced;
+                my %replaced;
 
                 while (@{$listp} > 0 && ${$listp}[$0] !~ /ADD|DELETE|REPLACE/) {
                     $str = $self->_rewriteParse($rule->{match}, $substitution, "$attr: ".shift @{$listp}, \%rwcache);
@@ -215,14 +230,16 @@ sub pre_modify
                         return -1;
                     } elsif ($str) {
                         foreach my $line (split(/\n/, $str)) {
-                            my $value;
-                            ($attr, $value) = split(/: */, $line);
-                            push(@replaced, $value);
+                            my ($rwattr, $value) = split(/: */, $line);
+                            if (!defined($replaced{$rwattr})) {
+                                @{$replaced{$rwattr}} = ();
+                            }
+                            push(@{$replaced{$rwattr}}, $value);
                         }
                     }
                 }
-                if (@replaced) {
-                    push(@mod_list, ($action, $attr, @replaced));
+                foreach my $rwattr (keys %replaced) {
+                    push(@mod_list, ($action, $rwattr, @{$replaced{$rwattr}}));
                 }
             }
             @{$listp} = @mod_list;
@@ -248,9 +265,13 @@ sub pre_add
 
     foreach my $rule (@{$conf->{rewrite}}) {
         if ($rule->{context} eq 'request') {
+            if (defined($rule->{dn}) && ${$dnp} !~ /$rule->{dn}/i) {
+                next;
+            }
+
             my %rwcache;
             my $substitution = $rule->{substitution};
-            $substitution =~ s/%0/${$dnp}\n${$entryStrp}/;
+            $substitution =~ s/%0/${$dnp}\n${$entryStrp}[0]/;
 
             my $str = $self->_rewriteParse($rule->{match}, $substitution, ${$dnp}, \%rwcache);
             if (!$str) {
@@ -290,6 +311,10 @@ sub pre_modrdn
 
     foreach my $rule (@{$conf->{rewrite}}) {
         if ($rule->{context} eq 'request') {
+            if (defined($rule->{dn}) && ${$dnp} !~ /$rule->{dn}/i) {
+                next;
+            }
+
             my %rwcache;
             my $substitution = $rule->{substitution};
             $substitution =~ s/%0/${$dnp}\n${$argsp}[0]/;
@@ -329,6 +354,10 @@ sub pre_delete
 
     foreach my $rule (@{$conf->{rewrite}}) {
         if ($rule->{context} eq 'request') {
+            if (defined($rule->{dn}) && ${$dnp} !~ /$rule->{dn}/i) {
+                next;
+            }
+
             my $substitution = $rule->{substitution};
             $substitution =~ s/%0/${$dnp}/;
 
