@@ -147,7 +147,6 @@ sub _do_modify
     my ($type, $dnp, $listp) = @_;
     my $conf = $self->{_config};
     my $dn = ${$dnp};
-    my @list = @{$listp};
     my $rc = 0;
     my $match = 0;
 
@@ -164,6 +163,7 @@ sub _do_modify
         }
 
         my @info;
+        my @list = @{$listp};
         while (@list > 0) {
             my $action = shift @list;
             my $attr = lc(shift @list);
@@ -201,24 +201,28 @@ sub _do_modify
             next;
         }
 
-        my $modinfo = join('#', @info);
+        my $modinfo = "dn=$dn#".join('#', @info);
         foreach my $script (@{$rule->{op}{modify}->{script}}) {
-            my @argv = split(/\s+/, $script);
-            for (my $i = 0; $i < @argv; $i++) {
-                $argv[$i] =~ s/%r/$rdn_val/;
-                $argv[$i] =~ s/%i/$modinfo/;
-            }
+            my $cmd = $script;
+            $cmd =~ s/%r/$rdn_val/g;
+            $cmd =~ s/%i/$modinfo/g;
 
-            $rc = system(@argv);
+            $rc = system($cmd);
             if ($rc) {
-                $self->log(level => 'error', message => "Script $argv[0]($dn) in modifying failed");
-                return $rc;
+                $self->log(level => 'error', message => "Script $script($dn) in modify failed");
+                if ($rc != 1) {
+                    return $rc;
+                }
             } else {
-                $self->log(level => 'info', message => "Script $argv[0]($dn) in modifying succeeded");
+                $self->log(level => 'info', message => "Script $script($dn) in modify succeeded");
             }
         }
     }
 
+    if ($rc == 1) {
+        $rc = 0;
+    }
+
     return $rc;
 }
 
@@ -241,6 +245,23 @@ sub _do_add
 
     my ($rdn_val) = ($dn =~ /^[^=]+=([^,]+),/);
 
+    my @info;
+    my (@line) = split(/\n/, $entryStr);
+    while (@line > 0) {
+        my ($attr, $values) = split(/: /, shift(@line));
+        if ($attr =~ /^structuralobjectclass$/i) {
+            last;
+        }
+
+        while ($line[0] =~ /^$attr: /) {
+            $line[0] =~ s/^$attr: //;
+            $values = "$values+$line[0]";
+            shift @line;
+        }
+        push(@info, "$attr=$values");
+    }
+    my $addinfo = "dn=$dn#".join('#', @info);
+
     foreach my $rule (@{$conf->{execrule}}) {
         if ($type ne $rule->{type}) {
             next;
@@ -256,39 +277,27 @@ sub _do_add
             next;
         }
 
-        my @info;
-        my (@line) = split(/\n/, $entryStr);
-        while (@line > 0) {
-            my ($attr, $values) = split(/: /, shift(@line));
-            if ($attr =~ /^structuralobjectclass$/i) {
-                last;
-            }
-
-            while ($line[0] =~ /^$attr: /) {
-                $line[0] =~ s/^$attr: //;
-                $values = "$values+$line[0]";
-                shift @line;
-            }
-            push(@info, "$attr=$values");
-        }
-
-        my ($addinfo) = join('#', @info);
         foreach my $script (@{$rule->{op}{add}->{script}}) {
-            my @argv = split(/\s+/, $script);
-            for (my $i = 0; $i < @argv; $i++) {
-                $argv[$i] =~ s/%r/$rdn_val/;
-                $argv[$i] =~ s/%i/$addinfo/;
-            }
+            my $cmd = $script;
+            $cmd =~ s/%r/$rdn_val/g;
+            $cmd =~ s/%i/$addinfo/g;
 
-            $rc = system(@argv);
+            $rc = system($cmd);
             if ($rc) {
-                $self->log(level => 'error', message => "Script $argv[0]($dn) in adding failed");
+                $self->log(level => 'error', message => "Script $script($dn) in add failed");
+                if ($rc != 1) {
+                    return $rc;
+                }
             } else {
-                $self->log(level => 'info', message => "Script $argv[0]($dn) in adding succeeded");
+                $self->log(level => 'info', message => "Script $script($dn) in add succeeded");
             }
         }
     }
 
+    if ($rc == 1) {
+        $rc = 0;
+    }
+
     return $rc;
 }
 
@@ -306,9 +315,11 @@ sub _do_delete
     my ($type, $dnp) = @_;
     my $conf = $self->{_config};
     my $dn = ${$dnp};
+    my $rc = 0;
 
     my ($rdn_val) = ($dn =~ /^[^=]+=([^,]+),/);
 
+    my $info = "dn=$dn";
     foreach my $rule (@{$conf->{execrule}}) {
         if ($type !~ /$rule->{type}/i) {
             next;
@@ -324,23 +335,29 @@ sub _do_delete
             }
 
             foreach my $script (@{$rule->{op}{$op}->{script}}) {
-                my @argv = split(/\s+/, $script);
-                for (my $i = 0; $i < @argv; $i++) {
-                    $argv[$i] =~ s/%r/$rdn_val/;
-                }
+                my $cmd = $script;
+                $cmd =~ s/%r/$rdn_val/g;
+                $cmd =~ s/%i/$info/g;
 
-                my $rc = system(@argv);
+                $rc = system($cmd);
                 if ($rc) {
-                    $self->log(level => 'error', message => "Script $argv[0]($dn) in deleting failed");
+                    $self->log(level => 'error', message => "Script $script($dn) in deleting failed");
+                    if ($rc != 1) {
+                        return $rc;
+                    }
                 } else {
-                    $self->log(level => 'info', message => "Script $argv[0]($dn) in deleting succeeded");
+                    $self->log(level => 'info', message => "Script $script($dn) in deleting succeeded");
                 }
             }
             next;
         }
     }
 
-    return 0;
+    if ($rc == 1) {
+        $rc = 0;
+    }
+
+    return $rc;
 }
 
 =head1 SEE ALSO
