@@ -36,7 +36,7 @@ sub new
     bless $this, $class;
 
     $this->{suffix} = $suffix;
-    $this->{contentry} = $contentry;
+    $this->{contentrystr} = $contentry;
 
     return $this;
 }
@@ -193,13 +193,13 @@ sub search
 
     my ($rc, @objs) = $self->_searchObjects($base, $scope);
     if ($rc) {
-        return $rc;
+        return ($rc, @match_entries);
     }
 
     # get entry of data container
     if ($base =~ /^$self->{suffix}$/) {
         if ($scope != 1) {
-            my $entry = $self->{contentry};
+            my $entry = $self->{contentrystr};
             if ($self->parseFilter($filter, $entry)) {
                 push (@match_entries, $entry);
                 $sizeLim--;
@@ -582,13 +582,13 @@ sub parseFilter
 
 =pod
 
-=head2 containerEntry($basedn, $conf)
+=head2 buildEntryStr($basedn, $conf)
 
 get information of container entry.
 
 =cut
 
-sub containerEntry
+sub buildEntryStr
 {
     my $self = shift;
     my ($basedn, $conf) = @_;
@@ -641,35 +641,43 @@ sub _checkConfig
     }
 
     if (defined($conf->{object})) {
+        $self->{object} = {};
         foreach my $oname (keys %{$conf->{object}}) {
-            my $obj = $conf->{object}{$oname};
+            my $oconf = $conf->{object}{$oname};
             my $entry;
 
-            # set containers
-            if (!defined($obj->{container})) {
+            if (!defined($oconf->{container})) {
                 $self->log(level => 'alert', message => "$oname object container doesn't exist");
                 return 1;
             }
-            if (defined($obj->{container}[0]->{oname})) {
-                $conf->{object}{$obj->{container}[0]->{oname}[0]}->{child} = $obj;
-            } elsif (!($entry = $self->containerEntry($self->{suffix}, $obj->{container}[0]))) {
+            $self->{object}{$oname}->{conf} = $oconf;
+
+            # set container
+            if (defined($oconf->{container}[0]->{oname})) {
+                $self->{object}{$oname}->{parent} = $oconf->{container}[0]->{oname}[0];
+                $self->{object}{$self->{object}{$oname}->{parent}}->{child} = $self->{object}{$oname};
+            } elsif (!($entry = $self->buildEntryStr($self->{suffix}, $oconf->{container}[0]))) {
                 $self->log(level => 'alert', message => "$oname object container entry is invalid");
                 return 1;
             } else {
-                $conf->{object}{$oname.'_container'}->{contentry} = $entry;
+                $self->{object}{$oname.'_container'}->{entrystr} = $entry;
                 # normalize suffix
-                $conf->{object}{$oname.'_container'}->{suffix} = lc($obj->{container}[0]->{rdn}[0].",".$self->{suffix});
+                $self->{object}{$oname.'_container'}->{suffix} = lc($oconf->{container}[0]->{rdn}[0].','.$self->{suffix});
                 # set child object
-                $conf->{object}{$oname.'_container'}->{child} = $obj;
-                $obj->{container}[0]->{oname}[0] = $oname.'_container';
+                $self->{object}{$oname.'_container'}->{child} = $self->{object}{$oname};
+                $self->{object}{$oname}->{parent} = $oname.'_container';
+            }
+
+            if (defined($oconf->{entry})) {
+                next;
             }
 
             # check id definition
-            if (!defined($obj->{id}) || !defined($obj->{id}[0]->{column})) {
+            if (!defined($oconf->{id}) || !defined($oconf->{id}[0]->{column})) {
                 $self->log(level => 'alert', message => "Id definition is invalid");
             }
 
-            foreach my $attr (keys(%{$obj->{attr}})) {
+            foreach my $attr (keys(%{$oconf->{attr}})) {
                 # the attribute's name must be lowercase
                 if ($attr =~ /[A-Z]/) {
                     $self->log(level => 'alert', message => "Attribute's name must be lowercase: $attr");
@@ -677,30 +685,41 @@ sub _checkConfig
                 }
 
                 # multibyte character code
-                if (defined($obj->{attr}{$attr}->{mbcode}) &&
-                    !($obj->{attr}{$attr}->{mbcode}[0] =~ /^(euc-jp|shiftjis)/)) {
+                if (defined($oconf->{attr}{$attr}->{mbcode}) &&
+                    !($oconf->{attr}{$attr}->{mbcode}[0] =~ /^(euc-jp|shiftjis)/)) {
                     $self->log(level => 'alert', message => "character code is invalid");
                     return 1;
                 }
             }
 
-            if (!defined($obj->{rdn}) || !defined($obj->{attr}{$obj->{rdn}[0]})) {
+            if (!defined($oconf->{rdn}) || !defined($oconf->{attr}{$oconf->{rdn}[0]})) {
                 $self->log(level => 'alert', message => "rdn of $oname object is invalid");
                 return 1;
             }
         }
 
         # check container's link
-        foreach my $oname (keys %{$conf->{object}}) {
-            my $obj = $conf->{object}{$oname};
+        foreach my $oname (keys %{$self->{object}}) {
+            my $obj = $self->{object}{$oname};
 
-            if (!defined($obj->{suffix}) && !defined($obj->{child})) {
+            if (!defined($obj->{suffix}) || !defined($obj->{child})) {
                 next;
             }
 
             my $leaf_exist = 0;
-            for (my $i = 0, my $child = $obj->{child}; $i < $maxLoopCount;
-                $i++, $child = $child->{child}) {
+            for (my $i = 0, my $parent = $obj, my $child = $obj->{child};
+                $i < $maxLoopCount;
+                    $i++, $parent = $parent->{child}, $child = $child->{child}) {
+                if (defined($child->{conf}->{entry})) {
+                    my $entry;
+                    my ($suffix) = ($parent->{entrystr} =~ /^dn: (.*)\n/);
+                    if (!($entry = $self->buildEntryStr($suffix, $child->{conf}->{entry}[0]))) {
+                        $self->log(level => 'alert', message => "container entry is invalid");
+                        return 1;
+                    }
+                    $child->{entrystr} = $entry;
+                }
+
                 if (!defined($child->{child})) {
                     $leaf_exist = 1;
                     last;
@@ -732,12 +751,12 @@ sub _getObject
     my $conf = $self->{_config};
     my $obj = undef;
 
-    if (!defined($conf->{object})) {
+    if (!defined($self->{object})) {
         return (LDAP_UNWILLING_TO_PERFORM, $obj, undef);
     }
 
-    foreach my $oname (keys %{$conf->{object}}) {
-        $obj = $conf->{object}{$oname};
+    foreach my $oname (keys %{$self->{object}}) {
+        $obj = $self->{object}{$oname};
 
         if (defined($obj->{suffix}) && $dn =~ /$obj->{suffix}$/i) {
             if ($dn =~ /^$obj->{suffix}$/i) {
@@ -753,10 +772,9 @@ sub _getObject
     }
 
     my (@rdn_list) = split(/,/, ($dn =~ /^[^,]+,(.*),?$obj->{suffix}$/i)[0]);
-    my $base = $obj->{suffix};
     my $key;
     for (my $base = $obj->{suffix}; defined($obj->{child});
-            $obj = $obj->{child}, $base = pop(@rdn_list).','.$base) {
+        $obj = $obj->{child}, $base = pop(@rdn_list).','.$base) {
         my $rc;
         my $entry;
 
@@ -770,11 +788,13 @@ sub _getObject
     }
 
     if (defined($obj->{child})) {
-        if ($dn !~ /^$obj->{child}->{rdn}[0]=/i) {
-            return (LDAP_UNWILLING_TO_PERFORM, $obj, undef);
+        my $child = $obj->{child};
+        if ((defined($child->{conf}->{rdn}) && $dn !~ /^$child->{conf}->{rdn}[0]=/i) ||
+            (defined($child->{entrystr}) && $child->{entrystr} !~ /^dn: $dn/i)) {
+            return (LDAP_NO_SUCH_OBJECT, $obj, undef);
         }
 
-        return (LDAP_SUCCESS, $obj->{child}, $key);
+        return (LDAP_SUCCESS, $child, $key);
     } else {
         return (LDAP_NO_SUCH_OBJECT, $obj, undef);
     }
@@ -793,9 +813,9 @@ sub _searchObjects
 
     if ($base =~ /^$self->{suffix}$/) {
         if ($scope != 0) {
-            foreach my $oname (keys %{$conf->{object}}) {
-                if (defined($conf->{object}{$oname}->{suffix})) {
-                    push(@objs, [$conf->{object}{$oname}, undef]);
+            foreach my $oname (keys %{$self->{object}}) {
+                if (defined($self->{object}{$oname}->{suffix})) {
+                    push(@objs, [$self->{object}{$oname}, undef]);
                 }
             }
         }
@@ -816,12 +836,12 @@ sub _baseSearch
     my $self = shift;
     my ($obj, $pkey, $base, $scope, $deref, $sizeLim, $timeLim, $filter) = @_;
 
-    if (!($base =~ /$obj->{suffix}$/i)) {
+    if ($base !~ /$obj->{suffix}$/i) {
         return (LDAP_NO_SUCH_OBJECT, undef, undef);
     }
 
-    if ($base =~ /^$obj->{suffix}$/i) {
-        my $entry = $obj->{contentry};
+    if (defined($obj->{entrystr})) {
+        my $entry = $obj->{entrystr};
         if ($self->parseFilter($filter, $entry) && $scope != 1 && $sizeLim) {
             return (LDAP_SUCCESS, undef, $entry);
         } else {
@@ -858,29 +878,42 @@ sub _childSearch
     my $ckeys;
     my $rc = LDAP_SUCCESS;
 
-    if ($scope == 0 || !defined($obj->{child}) || !$pkey && !defined($obj->{contentry})) {
+    if ($scope == 0 || !defined($obj->{child}) || !$pkey && !defined($obj->{entrystr})) {
         return ($rc, \@match_keys, ());
     }
 
     my @entries;
     my $keys;
-    ($rc, $keys, @entries) = $self->_objSearch($obj->{child}, $pkey, $base, $sizeLim, $filter);
-    push(@match_entries, @entries);
-    push(@match_keys, @{$keys});
+    if (defined($obj->{child}->{entrystr})) {
+        my $entry = $obj->{child}->{entrystr};
+        if ($self->parseFilter($filter, $entry) && $scope != 0 && $sizeLim) {
+            push(@match_entries, $entry);
+            push(@match_keys, undef);
+        }
+    } else {
+        ($rc, $keys, @entries) = $self->_objSearch($obj->{child}, $pkey, $base, $sizeLim, $filter);
+        push(@match_entries, @entries);
+        push(@match_keys, @{$keys});
+    }
 
     if ($scope == 1 || !defined($obj->{child}->{child}) || $rc) {
         return ($rc, \@match_keys, @match_entries);
     }
     $sizeLim = $sizeLim - @match_entries;
 
-    ($rc, $ckeys, @children) = $self->_objSearch($obj->{child}, $pkey, $base, -1, undef);
-    if ($rc) {
-        return ($rc, \@match_keys, @match_entries);
+    if (defined($obj->{child}->{entrystr})) {
+        push(@children, $obj->{child}->{entrystr});
+        my @tmp_keys = (undef);
+        $ckeys = \@tmp_keys;
+    } else {
+        ($rc, $ckeys, @children) = $self->_objSearch($obj->{child}, $pkey, $base, -1, undef);
+        if ($rc) {
+            return ($rc, \@match_keys, @match_entries);
+        }
     }
 
     for (my $i = 0; $i < @{$ckeys}; $i++) {
         my ($childdn) = ($children[$i] =~ /^dn: (.*)\n/);
-
         ($rc, $keys, @entries) = $self->_childSearch($obj->{child}, ${$ckeys}[$i], $childdn, $scope, $deref, $sizeLim, $timeLim, $filter);
         if ($rc) {
             last;
@@ -923,12 +956,12 @@ sub _getParentDn
     if (defined($obj->{suffix})) {
         return $obj->{suffix};
     }
-        
-    if (!defined($obj->{container}[0]->{oname})) {
+
+    if (!defined($obj->{parent})) {
         return undef;
     }
 
-    my $pobj = $conf->{object}{$obj->{container}[0]->{oname}[0]};
+    my $pobj = $self->{object}{$obj->{parent}};
     if (defined($pobj->{suffix})) {
         return $pobj->{suffix};
     }
