@@ -174,7 +174,7 @@ sub _objSearch
             }
 
             # get all values of the entry
-            $entry = "dn: $oconf->{rdn}[0]=$rdn_val,$suffix\n";
+            my $rdn = "$oconf->{rdn}[0]=$rdn_val";
             foreach my $oc (@{$oconf->{oc}}) {
                 $entry = $entry."objectclass: $oc\n";
             }
@@ -191,12 +191,17 @@ sub _objSearch
                         last;
                     }
 
-                    $entry = $entry.$values;
+                    if ($values) {
+                        $entry = $entry.$values;
+                    }
                 }
             }
 
+            $rdn = decode($conf->{mbcode}[0], $rdn);
             $entry = decode($conf->{mbcode}[0], $entry);
 
+            $entry = "dn: $rdn,$suffix\n$entry";
+
             # parse filter
             if ($self->parseFilter($filter, encode('utf8', $entry))) {
                 push(@match_entries, $self->_pwdFormat($entry));
@@ -487,11 +492,6 @@ sub _objAdd
                 }
             }
 
-            # add the link with container
-            if (defined($oconf->{container}) && defined($oconf->{container}[0]->{idcolumn})) {
-                $data[$oconf->{container}[0]->{idcolumn}[0]] = $pkey;
-            }
-
             # add storage-specific information
             foreach my $strginfo (@{$oconf->{strginfo}}) {
                 my $value = $self->_getStrgInfoValue($strginfo, $dn, $entryStr);
@@ -501,6 +501,11 @@ sub _objAdd
                 }
             }
 
+            # add the link with container
+            if (defined($oconf->{container}) && defined($oconf->{container}[0]->{idcolumn})) {
+                $data[$oconf->{container}[0]->{idcolumn}[0]] = $pkey;
+            }
+
             print $file join($dlm, @data)."\n";
         }
     }
@@ -724,8 +729,10 @@ sub _getAttrValues
                         $value =~ s/%$coln/$data[$coln]/g;
                     }
 
-                    $attrStr = $attrStr."$attr: $value\n";
-                    splice(@keys, $i, 1);
+                    if ($value) {
+                        $attrStr = $attrStr."$attr: $value\n";
+                        splice(@keys, $i, 1);
+                    }
        	            last;
                 }
 	    }
