@@ -79,6 +79,7 @@ sub bind
     my $conf = $self->{_config};
 
     $binddn =~ s/$self->{suffix}$/$conf->{nc}/i;
+    $binddn = decode('utf8', $binddn);
 
     if ($self->_getConnect()) {
         return LDAP_SERVER_DOWN;
@@ -138,6 +139,7 @@ sub search
     }
 
     $base =~ s/$self->{suffix}$/$conf->{nc}/i;
+    $base = decode('utf8', $base);
     $filterStr =~ s/$self->{suffix}(\)*)/$conf->{nc}$1/gi;
     $filterStr = encode('utf8', $filterStr);
     my $msg = $self->{ldap}->search(base => $base, scope => $scope, deref => $deref, sizeLimit => $sizeLim, timeLimit => $timeLim, filter => $filterStr);
@@ -195,6 +197,7 @@ sub compare
     my $conf = $self->{_config};
 
     $dn =~ s/$self->{suffix}$/$conf->{nc}/i;
+    $dn = decode('utf8', $dn);
 
     my ($key, $val) = split(/=/, $avaStr);
 
@@ -237,6 +240,7 @@ sub modify
     my $conf = $self->{_config};
 
     $dn =~ s/$self->{suffix}$/$conf->{nc}/i;
+    $dn = decode('utf8', $dn);
 
     my @changes;
     while ( @list > 0) {
@@ -311,6 +315,7 @@ sub add
     my $conf = $self->{_config};
 
     $dn =~ s/$self->{suffix}$/$conf->{nc}/i;
+    $dn = decode('utf8', $dn);
     $entryStr = decode('utf8', $entryStr);
 
     my %attrs;
@@ -369,6 +374,8 @@ sub modrdn
     my $conf = $self->{_config};
 
     $dn =~ s/$self->{suffix}$/$conf->{nc}/i;
+    $dn = decode('utf8', $dn);
+    $newrdn = decode('utf8', $newrdn);
 
     if ($self->_getConnect()) {
         return LDAP_SERVER_DOWN;
@@ -396,6 +403,7 @@ sub delete
     my $conf = $self->{_config};
 
     $dn =~ s/$self->{suffix}$/$conf->{nc}/i;
+    $dn = decode('utf8', $dn);
 
     if ($self->_getConnect()) {
         return LDAP_SERVER_DOWN;
