@@ -159,6 +159,50 @@ sub _freeConnect
     }
 }
 
+sub _checkConfig
+{
+    my $self = shift;
+    my $conf = $self->{_config};
+    my $rc = 0;
+
+    $rc = $self->SUPER::_checkConfig();
+    if ($rc) {
+        return $rc;
+    }
+
+    foreach my $oname (keys %{$conf->{object}}) {
+        my $oconf = $conf->{object}{$oname};
+
+        # check container fromtbls
+        if (defined($oconf->{container}[0]->{oname})) {
+            if (defined($oconf->{container}[0]->{fromtbls})) {
+                my $poconf = $self->{object}{$oconf->{container}[0]->{oname}[0]}->{conf};
+                if (",$oconf->{container}[0]->{fromtbls}[0]," =~
+                    /,$poconf->{table}[0], */) {
+                    $self->log(level => 'alert', message => "fromtbls($oconf->{container}[0]->{fromtbls}[0]) mustn't include container object table");
+                    return 1;
+                }
+            }
+        }
+
+        foreach my $attr (keys %{$oconf->{attr}}) {
+            # check attribute fromtbls
+            if (defined($oconf->{attr}{$attr}->{oname})) {
+                if (defined($oconf->{attr}{$attr}->{fromtbls})) {
+                    my $aoconf = $self->{object}{$oconf->{attr}{$attr}->{oname}[0]}->{conf};
+                    if (",$oconf->{attr}{$attr}->{fromtbls}[0]," =~
+                        /,$aoconf->{table}[0], */) {
+                        $self->log(level => 'alert', message => "fromtbls($oconf->{attr}{$attr}->{fromtbls}[0]) mustn't include attribute object table");
+                        return 1;
+                    }
+                }
+            }
+        }
+    }
+
+    return 0;
+}
+
 =pod
 
 =head2 _objSearch($obj, $parent, $suffix, $sizeLim, $filter)
@@ -198,12 +242,14 @@ sub _objSearch
 
         # entries below suffix
         if ($pkey && defined($oconf->{container}) && !defined($oconf->{container}[0]->{rdn})) {
-            my $pobj = $conf->{object}{$oconf->{container}[0]->{oname}[0]};
+            my $poconf = $self->{object}{$oconf->{container}[0]->{oname}[0]}->{conf};
+
+            $from = "$from, $poconf->{table}[0]";
             if (defined($oconf->{container}[0]->{fromtbls})) {
-                $from = "$from,$oconf->{container}[0]->{fromtbls}[0]";
+                $from = "$from, $oconf->{container}[0]->{fromtbls}[0]";
             }
 
-            $where = "$where and $pobj->{table}[0].$pobj->{id}[0]->{column}[0] = \'$pkey\'";
+            $where = "$where and $poconf->{table}[0].$poconf->{id}[0]->{column}[0] = \'$pkey\'";
             if (defined($oconf->{container}[0]->{joinwhere})) {
                 $where = "$where and $oconf->{container}[0]->{joinwhere}[0]";
             }
@@ -213,7 +259,6 @@ sub _objSearch
 
         # multibyte value
         $sql = encode($conf->{mbcode}[0], decode('utf8', $sql));
-
         my $sth = $self->{db}->prepare($sql);
         if (!$sth->execute) {
             $self->log(level => 'error', message => "Searching by \"$sql\" failed: ".$sth->errstr);
@@ -237,7 +282,7 @@ sub _objSearch
                 next;
             }
 
-            $entry = "dn: $oconf->{rdn}[0]=".shift(@data).",$suffix\n";
+            my $rdn = "$oconf->{rdn}[0]=".shift(@data);
 
             my $j = 0;
             foreach my $attr ('objectclass', keys %{$oconf->{attr}}) {
@@ -262,20 +307,30 @@ sub _objSearch
                         @match_entries = ();
                         last DO;
                     }
- 
-                    $entry = $entry.$values;
+
+                    if ($values) {
+                        $entry = $entry.$values;
+                    }
                 }
             }
 
             # multibyte value
             if ($conf->{dsn}[0] =~ /^DBI:Oracle/i && Encode::is_utf8($entry)) {
                 # add-hoc method for Oracle
-                Encode::_utf8_off($entry);
+                if (Encode::is_utf8($rdn)) {
+                    Encode::_utf8_off($rdn);
+                }
+                if (Encode::is_utf8($entry)) {
+                    Encode::_utf8_off($entry);
+                }
             }
 
+            Encode::from_to($rdn, $conf->{mbcode}[0], 'utf8');
+            $rdn = decode('utf8', $rdn);
             Encode::from_to($entry, $conf->{mbcode}[0], 'utf8');
             $entry = decode('utf8', $entry);
 
+            $entry = "dn: $rdn,$suffix\n$entry";
             push(@match_entries, $self->_pwdFormat($entry));
             push(@match_keys, $key);
         }
@@ -322,6 +377,7 @@ sub _objModify
             $rc = LDAP_NO_SUCH_OBJECT;
             last DO;
         }
+        $entry = encode('utf8', $entry);
 
         while ( @list > 0 ) {
             my $action = shift @list;
@@ -609,17 +665,6 @@ sub _objAdd
             $key = shift @{$keys};
         }
 
-        # add the values in the attribute's table
-        foreach my $attr (keys %attrs) {
-            foreach my $value (@{$attrs{$attr}}) {
-                if ($rc = $self->_addAttrValues($oconf, $key, $attr, $value)) {
-                    $self->log(level => 'error', message => "Adding values of $attr failed");
-                    $rc = LDAP_OPERATIONS_ERROR;
-                    last DO;
-                }
-            }
-        }
-
         # add the storage-specific information
         for (my $i = 0; $i < @{$oconf->{strginfo}}; $i++) {
             if (!defined(${$oconf->{strginfo}}[$i]->{addproc})) {
@@ -644,6 +689,17 @@ sub _objAdd
             }
         }
 
+        # add the values in the attribute's table
+        foreach my $attr (keys %attrs) {
+            foreach my $value (@{$attrs{$attr}}) {
+                if ($rc = $self->_addAttrValues($oconf, $key, $attr, $value)) {
+                    $self->log(level => 'error', message => "Adding values of $attr failed");
+                    $rc = LDAP_OPERATIONS_ERROR;
+                    last DO;
+                }
+            }
+        }
+
         # add the link with container
         if (defined($oconf->{container}) && defined($oconf->{container}[0]->{addproc})) {
             $sql = $oconf->{container}[0]->{addproc}[0];
@@ -705,6 +761,16 @@ sub _objDelete
         my $sql;
         my $sth;
 
+        # delete the values from the attribute's table
+        foreach my $attr (keys %{$oconf->{attr}}) {
+            if (defined($oconf->{attr}{$attr}) && defined($oconf->{attr}{$attr}->{delproc})) {
+                if ($rc = $self->_delAttrValues($oconf, $key, $attr, '')) {
+                    $rc = LDAP_OPERATIONS_ERROR;
+                    last DO;
+                }
+            }
+        }
+
         # delete the storage-specific information
         foreach my $strginfo (@{$oconf->{strginfo}}) {
             if (!defined($strginfo->{delproc})) {
@@ -729,16 +795,6 @@ sub _objDelete
             }
         }
 
-        # delete the values from the attribute's table
-        foreach my $attr (keys %{$oconf->{attr}}) {
-            if (defined($oconf->{attr}{$attr}) && defined($oconf->{attr}{$attr}->{delproc})) {
-                if ($rc = $self->_delAttrValues($oconf, $key, $attr, '')) {
-                    $rc = LDAP_OPERATIONS_ERROR;
-                    last DO;
-                }
-            }
-        }
-
         # delete the link with container
         if (defined($oconf->{container}) && defined($oconf->{container}[0]->{delproc})) {
             $sql = $oconf->{container}[0]->{delproc}[0];
@@ -796,7 +852,7 @@ sub _getParentRdn
 
     $selexpr = "$poconf->{table}[0].$poconf->{id}[0]->{column}[0],$poconf->{table}[0].$poconf->{attr}{$poconf->{rdn}[0]}->{column}[0]";
 
-    $from = $oconf->{table}[0];
+    $from = "$oconf->{table}[0], $poconf->{table}[0]";
     if (defined($oconf->{container}[0]->{fromtbls})) {
         $from = "$from,$oconf->{container}[0]->{fromtbls}[0]";
     }
@@ -807,7 +863,6 @@ sub _getParentRdn
     }
 
     my $sql = "select $selexpr from $from where $where";
-print STDERR "sql: $sql\n";
     my $sth = $self->{db}->prepare($sql);
     if (!$sth->execute) {
         $self->log(level => 'error', message => "Getting rdn by \"$sql\" failed: ".$sth->errstr);
@@ -828,20 +883,29 @@ sub _getAttrValues
     my $conf = $self->{_config};
     my $aobj = undef;
     my $aoconf = undef;
+    my $cobj = undef;
+    my $coconf = undef;
     my $attrStr = '';
     my $selexpr;
     my $from;
     my $where;
 
+    $from = $oconf->{table}[0];
     if (defined($oconf->{attr}{$attr}->{oname})) {
         $aobj = $self->{object}{$oconf->{attr}{$attr}->{oname}[0]};
         $aoconf = $aobj->{conf};
         $selexpr = "$aoconf->{table}[0].$aoconf->{id}[0]->{column}[0], $aoconf->{table}[0].$aoconf->{attr}{$aoconf->{rdn}[0]}->{column}[0]";
+        $from = "$from, $aoconf->{table}[0]";
+        if (defined($oconf->{attr}{$attr}->{cname})) {
+            $cobj = $self->{object}{$oconf->{attr}{$attr}->{cname}[0]};
+            $coconf = $cobj->{conf};
+            $selexpr = "$selexpr, $coconf->{table}[0].$coconf->{id}[0]->{column}[0], $coconf->{table}[0].$coconf->{attr}{$coconf->{rdn}[0]}->{column}[0]";
+            $from = "$from, $coconf->{table}[0]";
+        }
     } else {
         $selexpr = $oconf->{attr}{$attr}->{selexpr}[0];
     }
 
-    $from = $oconf->{table}[0];
     if (defined($oconf->{attr}{$attr}->{fromtbls})) {
         $from = "$from,$oconf->{attr}{$attr}->{fromtbls}[0]";
     }
@@ -863,9 +927,16 @@ sub _getAttrValues
         if (!@data) {
             next;
         } elsif ($aoconf) {
-            $attrStr = $attrStr."$attr: $aoconf->{rdn}[0]=$data[1],".$self->_getParentDn($aobj, $data[0])."\n";
-        } else {
-            $attrStr = $attrStr."$attr: $data[0]\n";
+            $attrStr = $attrStr."$attr: $aoconf->{rdn}[0]=$data[1],";
+            if ($coconf) {
+                $attrStr = $attrStr."$coconf->{rdn}[0]=$data[3],".$self->_getParentDn($cobj, $data[2])."\n";
+            } else {
+                $attrStr = $attrStr.$self->_getParentDn($aobj, $data[0])."\n";
+            }
+        } else{
+            if ($data[0]) {
+                $attrStr = $attrStr."$attr: $data[0]\n";
+            }
         }
     }
     $sth->finish;
@@ -893,7 +964,6 @@ sub _addAttrValues
             }
 
             my $attrkey;
-
             ($rc, $attrkey) =$self->_baseSearch($aobj, $attrpkey, $value, 0, 0, 1, 0, undef);
             if ($rc) {
                 $self->log(level => 'error', message => "Can't get id of $value in the table");
@@ -903,6 +973,26 @@ sub _addAttrValues
                 return 1;
             }
             $sql =~ s/\%a/$attrkey/g;
+
+            if (defined($oconf->{attr}{$attr}->{cname})) {
+                my $contdn;
+                ($contdn = $value) =~ s/^[^,]+,//;
+                my ($rc, $cobj, $contpkey) = $self->_getObject($contdn);
+                if ($rc) {
+                    return -1;
+                }
+
+                my $contkey;
+                ($rc, $contkey) =$self->_baseSearch($cobj, $contpkey, $contdn, 0, 0, 1, 0, undef);
+                if ($rc) {
+                    $self->log(level => 'error', message => "Can't get id of $contdn in the table");
+                    return -1;
+                } elsif (!$contkey) {
+                    $self->log(level => 'error', message => "Id of $contdn doesn't exist in the table");
+                    return 1;
+                }
+                $sql =~ s/\%c/$contkey/g;
+            }
         } else {
             $sql =~ s/\%a/$value/g;
         }
@@ -943,7 +1033,6 @@ sub _delAttrValues
                 }
 
                 my $attrkey;
-
                 ($rc, $attrkey) =$self->_baseSearch($aobj, $attrpkey, $value, 0, 0, 1, 0, undef);
                 if ($rc) {
 		    $self->log(level => 'error', message => "Can't get id of $value from the table");
@@ -953,6 +1042,26 @@ sub _delAttrValues
                     return 1;
                 }
                 $sql =~ s/\%a/$attrkey/g;
+
+                if (defined($oconf->{attr}{$attr}->{cname})) {
+                    my $contdn;
+                    ($contdn = $value) =~ s/^[^,]+,//;
+                    my ($rc, $cobj, $contpkey) = $self->_getObject($contdn);
+                    if ($rc) {
+                        return -1;
+                    }
+
+                    my $contkey;
+                    ($rc, $contkey) =$self->_baseSearch($cobj, $contpkey, $contdn, 0, 0, 1, 0, undef);
+                    if ($rc) {
+                        $self->log(level => 'error', message => "Can't get id of $contdn in the table");
+                        return -1;
+                    } elsif (!$contkey) {
+                        $self->log(level => 'error', message => "Id of $contdn doesn't exist in the table");
+                        return 1;
+                    }
+                    $sql =~ s/\%c/$contkey/g;
+                }
             } else {
                 $sql =~ s/\%a/$value/g;
             }
@@ -1074,7 +1183,7 @@ sub _filter2sql
         if (defined($oconf->{attr}{$attr}->{fromtbls})) {
             ${$from} = "${$from},$oconf->{attr}{$attr}->{fromtbls}[0]";
         }
-        ${$where} = "$oconf->{attr}{$attr}->{selexpr} $type \'$value\'";
+        ${$where} = "$oconf->{attr}{$attr}->{selexpr}[0] $type \'$value\'";
         if (defined($oconf->{attr}{$attr}->{joinwhere})) {
             ${$where} = "${$where} and $oconf->{attr}{$attr}->{joinwhere}[0]";
         }
